﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace ShockDraw
{
	public partial class frmMain : Form
	{
		[DllImport("sensor.dll")]
		public static extern void ShockproofGetAccelerometerData(ref AccData accData);

		[StructLayout(LayoutKind.Sequential)]
		public struct AccData
		{
			internal int status;
			internal short x;
			internal short y;
			internal ushort xx;
			internal ushort yy;
			internal sbyte c;
			internal ushort x0;
			internal ushort y0;
		}

		private readonly object lockBMP = new object();

		private Bitmap bmpBackground = null;

		private int iX = 0;
		private int iY = 0;

		private int iFlatX = 0;
		private int iFlatY = 0;

		private int iR = 0;
		private int iG = 0;
		private int iB = 0;

		AccData oShockData = new AccData();

		public frmMain()
		{
			InitializeComponent();
		}

		private void frmMain_Load(object sender, EventArgs e)
		{
			frmMain_Resize(null, null);

			ShockproofGetAccelerometerData(ref oShockData);
			iFlatX = oShockData.x;
			iFlatY = oShockData.y;

			tmrUpdate.Enabled = true;
			tmrUpdate.Start();
		}

		private void tmrUpdate_Tick(object sender, EventArgs e)
		{
			try
			{
				lock (lockBMP)
				{
					tmrUpdate.Stop();

					ShockproofGetAccelerometerData(ref oShockData);
					txtX.Text = oShockData.x.ToString();
					txtY.Text = oShockData.y.ToString();
					txtStatus.Text = oShockData.status.ToString();

					int iDeltaX = oShockData.y - iFlatY;
					int iDeltaY = oShockData.x - iFlatX;

					if (Math.Abs(iDeltaX) < 1)
						iDeltaX = 0;
					if (Math.Abs(iDeltaY) < 1)
						iDeltaY = 0;

					if (iDeltaX < -125)
						iDeltaX = -125;
					if (iDeltaX > 125)
						iDeltaX = 125;

					if (iDeltaY < -125)
						iDeltaY = -125;
					if (iDeltaY > 125)
						iDeltaY = 125;

					int iOldX = iX;
					int iOldY = iY;

					iX += (int)((float)10 * ((float)iDeltaX / 125.0f));
					iY += (int)((float)10 * ((float)iDeltaY / 125.0f));

					if (iX < 0)
						iX = 0;
					if (iX > this.Width - 1)
						iX = this.Width - 1;

					if (iY < 0)
						iY = 0;
					if (iY > this.Height - 1)
						iY = this.Height - 1;

					Graphics g = Graphics.FromImage(bmpBackground);

					iR += 1;
					if (iR > 255)
						iR = iR - 255;

					iG += 2;
					if (iG > 255)
						iG = iG - 255;

					iB += 3;
					if (iB > 255)
						iB = iB - 255;

					Pen oPen = new Pen(Color.FromArgb(iR, iG, iB), 5);
					g.DrawLine(oPen, iOldX, iOldY, iX, iY);

					g.Dispose();
					g = null;

					this.Invalidate();
				}

				return;
			}
			catch (Exception ex)
			{
				txtStatus.Text = ex.Message;
			}
			finally
			{
				tmrUpdate.Start();
			}
		}

		private void frmMain_Resize(object sender, EventArgs e)
		{
			lock (lockBMP)
			{
				bmpBackground = new Bitmap(this.Width, this.Height);

				iX = this.Width / 2;
				iY = this.Height / 2;

				this.BackgroundImage = bmpBackground;
				this.BackgroundImageLayout = ImageLayout.None;

				Graphics g = Graphics.FromImage(bmpBackground);
				g.Clear(Color.White);
				g.Dispose();
				g = null;
			}
		}
	}
}